/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef lswidget_included
#define lswidget_included

// uncomment to enable drag & drop
#define DRAGDROP

#include <wintv.h>

#include <qapp.h>
#include <qlist.h>
#include <qpixmap.h>
#include <qwidget.h>
//#include <qlistbox.h>
#include <qlistview.h> 
#include <qtooltip.h>
#include <qpopmenu.h>
#include <qbutton.h>
#include <qpushbutton.h> 
#include <qslider.h> // use KSlider in later versions of KDE

#include <kapp.h>
//#include <kpopmenu.h>
//#include <kslider.h>
#include <ktoolbar.h>
#include <ktopwidget.h>

#include <rcfile.h>
#include <channel.h>

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class myQListViewItem : public QListViewItem
{
public:
  myQListViewItem ( QListView * parent ) : QListViewItem ( parent ) {}
  myQListViewItem ( QListViewItem * parent ) : QListViewItem ( parent ) {}
  myQListViewItem ( QListView * parent, const char * i1, const char * i2= 0, const char * i3= 0 )
    : QListViewItem ( parent, i1, i2, i3 ) {}
 
  virtual const char* key ( int, bool ) const;

};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winSlider : public QSlider
{
public:
  winSlider ( QWidget * parent=0, const char * name=0 ):QSlider ( parent, name ) {}   
  // ~winSlider();

  void setSPopup(QPopupMenu *sp) { sPopup=sp; }

protected:
  /* ------------------------------------------------------------------------------*/
  void mouseReleaseEvent ( QMouseEvent * me);

private:
  QPopupMenu *sPopup;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winListBox : public QListView
{
  Q_OBJECT
public:
  winListBox ( QWidget * parent=0, const char * name=0 )
    : QListView ( parent,name ), 
      pressedButton( 0 ), oldY(0), itemWasMoved( false ), cursorChanged( false )
  {  setMouseTracking (true); }
  // ~winListBox();

  void setCPopup(QPopupMenu *sp) { cPopup=sp; }

protected:
#ifdef DRAGDROP
  void mouseReleaseEvent ( QMouseEvent * me);
  void mousePressEvent ( QMouseEvent * me);
#endif
  void mouseMoveEvent ( QMouseEvent * me);

  void rightButtonClicked ( QListViewItem *, const QPoint & pos, int);

signals:
  void moveItemSIGNAL( QListViewItem *, QListViewItem * );

private:
  QPopupMenu *cPopup;

  QListViewItem *toMoveItem, *lastVisItem;
  int pressedButton;
  int oldY;
  bool itemWasMoved;
  bool cursorChanged;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winWidget : public QWidget
{
  Q_OBJECT

public:
  winWidget(QWidget* parent = NULL,
	   const char* name = NULL,
	   int _sliderAtt=1);
  ~winWidget();

  void setSPopup(QPopupMenu *sp) {sPopup=sp;mixerSlider_right->setSPopup(sPopup);mixerSlider_left->setSPopup(sPopup);} 
  void setCPopup(QPopupMenu *cp) { cPopup=cp;channelList->setCPopup(cPopup); }

public slots:
  void setValue ( int vl, int vr );
  void splitSliderSLOT(int split);
  void insertItem ( const channel* info, bool ins );  
  void removeItem ( int index ); 
  void changeItem ( const char * text, int index ); 
  void setCurrentItem ( int index );  
  void centerCurrentItem ();  
  void clear();
  void muteSLOT(bool on);

protected slots:
  void leftSliderSLOT(int value);
  void rightSliderSLOT(int value);
  void highlightedSLOT(QListViewItem * item);
  void channelEditSelectSLOT(QListViewItem *);
  void moveItemSLOT( QListViewItem *, QListViewItem * );

protected:
  /* ------------------------------------------------------------------------------*/
  //void mouseReleaseEvent ( QMouseEvent * me);
  void resizeEvent( QResizeEvent * );

signals:
  void highlighted(int id);
  void valueChanged(int vl, int vr);
  void muteChanged(int mute);
  void channelEditSelectSIGNAL(int id);
  void moveItemSIGNAL( int from, int to);

private:
  winSlider* mixerSlider_right;
  winSlider* mixerSlider_left;

  winListBox* channelList;
  //myQListViewItem *rootItem;

  QList<QListViewItem> aChannelList;

  QPopupMenu *sPopup,*cPopup;
  QPushButton *lsbutton;
  //QButton *lsbutton;

  QPixmap ls_xpm;

  int sliderAtt;
  int volume_right,volume_left;

  bool inserting;
  bool fixedSliders;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winTVChannelWidget : public KTopLevelWidget
{
  Q_OBJECT

public:
  winTVChannelWidget(QWidget* parent = NULL,const char* name = NULL, rcfile *_rc=NULL );
  ~winTVChannelWidget();

  void setPopups(QPopupMenu *sp,QPopupMenu *cp);

public slots:
  void newChannelListSLOT(QList<channel> *clist);
  void activeChannelChangedSLOT();
  void volumeChangedSLOT(int vl,int vr);
  void toggleSplitSlidersSLOT();
  void toggleVisibilitySLOT( QPoint *to );

protected slots:
  void buttonsSLOT(int);
  void volumeChangedEmitSLOT(int, int);
  void channelSelectedEmitSLOT(int);
  void moveItemSLOT(int from, int to);

signals:
  void channelSelectedSIGNAL();
  void addChannelSIGNAL();
  void editChannelSIGNAL();
  void deleteChannelSIGNAL();
  void copyChannelSIGNAL();
  void pasteChannelSIGNAL();
  void volumeChangedSIGNAL(int, int);
  void toggleSplitSlidersSIGNAL();

private:
  winWidget *channelbox;
  KToolBar *toolbar;

  QPixmap  pm;
  QPixmap  ch_add_xpm, ch_edit_xpm, ch_delete_xpm, sa_xpm, cs_xpm;
  
  rcfile *rc;

  QList<channel> *clistp;

  bool inserting;
};

#endif





